/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * ControlPanel.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import javax.swing.*;
import javax.swing.JComponent.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.*;

public class ControlPanel extends JPanel implements ActionListener, ChangeListener, ItemListener {

	private int timerDelay = 50; // in ms

	PlayController controller;
	MetaMix parent;
	
	JButton startJButton, stopJButton, skipJButton, backJButton, aboutJButton, helpJButton;
	JCheckBox muteBox;
	JSlider volSlider;
	JLabel chunkLabel, status, modeLabel, nLabel, fileLabel, mixLabel;
	javax.swing.Timer timer;
	Counter counter;
        boolean buttonSaysPlay = true;

	ControlPanel (PlayController pc, MetaMix p) {
	    controller = pc;
	    parent = p;
	
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    add(Box.createVerticalGlue());
	    add(Box.createRigidArea(new Dimension(0,10)));
	    
	    JPanel sourcePanel = new JPanel();
	    sourcePanel.setLayout(new BoxLayout(sourcePanel, BoxLayout.X_AXIS));
	    sourcePanel.add(Box.createRigidArea(new Dimension(20,0)));
	    JLabel label = new JLabel("Source: ", JLabel.LEFT);
	    sourcePanel.add(label);
	    fileLabel = new JLabel("(none)", JLabel.LEFT);
	    sourcePanel.add(fileLabel);
	    sourcePanel.add(Box.createHorizontalGlue());
	    add(sourcePanel);
	    
	    //add(Box.createRigidArea(new Dimension(0,5)));
	    
	    JPanel formPanel = new JPanel();
	    formPanel.setLayout(new BoxLayout(formPanel, BoxLayout.X_AXIS));
	    formPanel.add(Box.createRigidArea(new Dimension(20,0)));	    
	    label = new JLabel("Form: ", JLabel.LEFT);
	    formPanel.add(label);
	    modeLabel = new JLabel(Traverser.getName(Traverser.DEFAULT_SEQUENCE), JLabel.LEFT);
	    formPanel.add(modeLabel);
            nLabel = new JLabel(" [f(0) = 0]", JLabel.LEFT);
	    formPanel.add(nLabel);
	    formPanel.add(Box.createHorizontalGlue());
	    add(formPanel);
	    
	    //add(Box.createRigidArea(new Dimension(0,5)));
	    
    	    JPanel counterPanel = new JPanel();
	    counterPanel.setLayout(new BoxLayout(counterPanel, BoxLayout.X_AXIS));
	    counterPanel.add(Box.createRigidArea(new Dimension(20,0)));
	    mixLabel = new JLabel("Mix: (stopped)", JLabel.LEFT);
	    counterPanel.add(mixLabel);
	    counterPanel.add(Box.createRigidArea(new Dimension(5,0)));
	    counter = new Counter(controller, parent);
	    counter.repaint();
	    counterPanel.add(counter);
	    counterPanel.add(Box.createRigidArea(new Dimension(20,0)));
	    counterPanel.add(Box.createHorizontalGlue());
	    add(counterPanel);
	    
	    add(Box.createRigidArea(new Dimension(0,15)));

	    JPanel volPanel = new JPanel();
	    volPanel.setLayout(new BoxLayout(volPanel, BoxLayout.X_AXIS));
	    volPanel.add(Box.createHorizontalGlue());
	    
	    label = new JLabel("volume", JLabel.RIGHT);
	    volPanel.add(label);
	    
	    volSlider = new JSlider(0, 150, (int) (MetaMix.DEFAULT_VOLUME * 100));
	    volSlider.setPaintTicks(false);
	    volSlider.setPaintLabels(false);
	    volSlider.setSnapToTicks(false);
	    volSlider.addChangeListener(this);
	    volPanel.add(volSlider);
		 
	    volPanel.add(Box.createRigidArea(new Dimension(20,0)));

	    muteBox = new JCheckBox("mute", false);
	    muteBox.addItemListener(this);
	    volPanel.add(muteBox);
	    
	    volPanel.add(Box.createHorizontalGlue());
	    add(volPanel);
	    add(Box.createRigidArea(new Dimension(0,10)));

	    JPanel playPanel = new JPanel();
	    playPanel.setLayout(new BoxLayout(playPanel, BoxLayout.X_AXIS));
	    playPanel.add(Box.createHorizontalGlue());

            try {
                backJButton = new JButton(new ImageIcon(this.getClass().getResource("/html/rew.gif")));
            } catch (Exception e) {
                backJButton = new JButton("-->");
            }
	    backJButton.setActionCommand("back");
            backJButton.setMnemonic(KeyEvent.VK_B); // alt-b
            backJButton.setMargin(new Insets(5,5,5,5));
	    backJButton.addActionListener(this);
	    playPanel.add(backJButton);
            
	    playPanel.add(Box.createRigidArea(new Dimension(10,0)));
            
            try {
                startJButton = new JButton(new ImageIcon(this.getClass().getResource("/html/play.gif")));
            } catch (Exception e) {
                startJButton = new JButton("play");
            }
            startJButton.setActionCommand("start");
            startJButton.setMargin(new Insets(5,5,5,5));
            startJButton.setMnemonic(KeyEvent.VK_P); // alt-p
	    startJButton.addActionListener(this);
	    playPanel.add(startJButton);
	    
	    playPanel.add(Box.createRigidArea(new Dimension(10,0)));
	    
            try {
                stopJButton = new JButton(new ImageIcon(this.getClass().getResource("/html/stop.gif")));
            } catch (Exception e) {
                startJButton = new JButton("stop");
            }
            stopJButton.setActionCommand("stop");
            stopJButton.setMnemonic(KeyEvent.VK_S); // alt-s
            stopJButton.setMargin(new Insets(5,5,5,5));
	    stopJButton.addActionListener(this);
	    playPanel.add(stopJButton);
	    
	    playPanel.add(Box.createRigidArea(new Dimension(10,0)));
	    
            try {
                skipJButton = new JButton(new ImageIcon(this.getClass().getResource("/html/ff.gif")));
            } catch (Exception e) {
                skipJButton = new JButton("-->");
            }
	    skipJButton.setActionCommand("skip");
            skipJButton.setMnemonic(KeyEvent.VK_F); // alt-f
            skipJButton.setMargin(new Insets(5,5,5,5));
	    skipJButton.addActionListener(this);
	    playPanel.add(skipJButton);
            
	    playPanel.add(Box.createRigidArea(new Dimension(10,0)));
            
	    aboutJButton = new JButton("about");
	    aboutJButton.setActionCommand("about");
	    aboutJButton.addActionListener(this);
	    playPanel.add(aboutJButton);
	    
	    playPanel.add(Box.createRigidArea(new Dimension(10,0)));
	    
	    helpJButton = new JButton("help");	
	    helpJButton.setActionCommand("help");
	    helpJButton.addActionListener(this);
	    playPanel.add(helpJButton);
	    
	    playPanel.add(Box.createHorizontalGlue());
	    add(playPanel);
	    add(Box.createRigidArea(new Dimension(0,10)));

	    timer = new javax.swing.Timer(timerDelay, this);
	    timer.setInitialDelay(0);
	    timer.setCoalesce(true);
		    
	    add(Box.createVerticalGlue());
	}
	
	public void setStartButton(String s) {
	    startJButton.setText(s);
	}
	
	public void setChunkInfo(SlavedPlayer sp) {
	    counter.setChunkInfo(sp);
	}
	
	public void setFile(String s) {
	    fileLabel.setText(s);
	}
	
	public void setMode(int index) {
	    modeLabel.setText(Traverser.getName(index));
	}
	
	public void setMixLabel(boolean playing) {
	    if (playing)
		mixLabel.setText("Mix:");
	    else
		mixLabel.setText("Mix: (stopped)");
	}
	
	// for alignment purposes
	public int getLabelHeight() {
	    return(mixLabel.getHeight());
	}
	
	// smaller values mean smoother animation
	public void setTimerDelay(int ms) {
	    timerDelay = ms;
	    timer.setDelay(ms);
	}
	
	// these functions update GUI only...
	public void stop() {
            try {
                startJButton.setIcon(new ImageIcon(this.getClass().getResource("/html/play.gif")));
            } catch (Exception e) {
                startJButton.setText("play");
            }
            buttonSaysPlay = true;
	    timer.stop();
	    setMixLabel(false);
	    counter.stop();
	}
	
	public void play() {
            try {
                startJButton.setIcon(new ImageIcon(this.getClass().getResource("/html/pause.gif")));
            } catch (Exception e) {
                startJButton.setText("pause");
            }
            buttonSaysPlay = false;
	    timer.start();
	    setMixLabel(true);
	}
	
	public void pause() {
            try {
                startJButton.setIcon(new ImageIcon(this.getClass().getResource("/html/play.gif")));
            } catch (Exception e) {
                startJButton.setText("play");
            }
            buttonSaysPlay = true;
	    timer.stop();	
	}
	
	public void actionPerformed(ActionEvent e) {
	    if (e.getSource() == timer) {
		if (counter != null)
		    counter.repaint();
	    } else if (e.getActionCommand().equals("stop")) {
		if (controller.stop())
		    stop();
	    } else if (e.getActionCommand().equals("start")) {
		if (buttonSaysPlay) {
		    if (controller.play())
			play();
		} else {
		    if (controller.pause())
			pause();
		}
	    } else if (e.getActionCommand().equals("skip")) {
		controller.skip();
            } else if (e.getActionCommand().equals("back")) {
		controller.back();
            } else if (e.getActionCommand().equals("about")) {
		new AboutBox();
	    } else if (e.getActionCommand().equals("help")) {
			parent.helpBox.showBox();
	    }
	}
	
	public void stateChanged(ChangeEvent event) {
	    JSlider s = (JSlider) event.getSource();
	    if (s == volSlider) { 
		int w = (int)s.getValue();
		controller.setVolume(w);
	    }
	}
	
	public void itemStateChanged(ItemEvent e) {
	    Object source = e.getItemSelectable();
	    if (source == muteBox) {
		if (e.getStateChange() == ItemEvent.DESELECTED)
		    controller.setMute(1);
		else
		    controller.setMute(0);
	    } 
	}
}


