/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * Counter.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import javax.swing.*;
import java.awt.*;
import java.awt.font.*;
import java.util.*;

public class Counter extends JPanel {

	PlayController controller;
	MetaMix parent;

	Vector chunkQ;

	Font font = null;
	FontRenderContext frc = null;
	Color bgcolor;

	Counter(PlayController pc, MetaMix p) {
		super();
		controller = pc;
		parent = p;
		bgcolor = parent.getBackground();

		chunkQ = new Vector(MetaMix.MAX_NUMLAYERS);
	}

	public void stop() {
		chunkQ.removeAllElements();
	}

	public Dimension getMinimumSize() {
		return (getPreferredSize());
	}

	public Dimension getPreferredSize() {
		FontMetrics fm = getGraphics().getFontMetrics();
		return (
			new Dimension(
				parent.controls.getWidth(),
				fm.getHeight() + fm.getMaxDescent()));
	}

	public void setChunkInfo(SlavedPlayer sp) {
		while (chunkQ.size() >= controller.getNumPlayers())
			chunkQ.removeElementAt(0);
		chunkQ.addElement(new ChunkPlayer(sp, sp.getCurChunk()));
		repaint();
	}

	public void paintComponent(Graphics g) {
		if (controller.triggerLocked())
			// don't update while chunks are shuffling around...
			return; // prevents some types of sudden flashes...

		Graphics2D g2 = (Graphics2D) g;

		super.paintComponent(g);

		// if hidden, there's no need to waste CPU cycles...
		if (!isShowing())
			return;

		String display;
		String curchunk;
		Color color;
		int minutes, seconds, time, x, width;

		for (int i = 0; i < chunkQ.size(); i++) {
			ChunkPlayer player = (ChunkPlayer) chunkQ.elementAt(i);
			SlavedPlayer slave = player.player;
			try {
				// time check is necessary b/c slaves whose reverse play
				// ends up at 0 don't correctly show up as not playing
				if (slave.playing() && (slave.getTime() != 0)) {
					// set the text
					time = slave.getTime() / slave.getScale();
					minutes = (time / 60);
					seconds = (time % 60);
					if (seconds < 10)
						display = player.chunk + minutes + ":0" + seconds + ")";
					else
						display = player.chunk + minutes + ":" + seconds + ")";

					// set the color
					if (!slave.getReverse())
						color =
							new Color(
								(int) (bgcolor.getRed()
									* (1 - slave.getActualVolume())),
								(int) (bgcolor.getGreen()
									* (1 - slave.getActualVolume())),
								(int) (bgcolor.getBlue()
									* (1 - slave.getActualVolume())));
					else
						color =
							new Color(
								(int) (bgcolor.getRed()),
								(int) (bgcolor.getGreen()
									* (1 - slave.getActualVolume())),
								(int) (bgcolor.getBlue()
									* (1 - slave.getActualVolume())));
					g2.setColor(color);

					// set the position
					width = this.getWidth() / (controller.getNumPlayers() + 2);
					x =
						(int) (width
							* (chunkQ.size() - 1 - i + slave.getPercDone()));

					// draw it!
					g2.drawString(display, x, parent.controls.getLabelHeight());
				}
			} catch (Exception e) {
				; // don't draw!
			}
		}
	}

	private class ChunkPlayer {

		public SlavedPlayer player;
		public String chunk;

		ChunkPlayer(SlavedPlayer sp, int ch) {
			player = sp;
			chunk = Integer.toString(ch) + " (";
		}
	}
}
