/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * FormPanel.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import javax.swing.*;
import javax.swing.JComponent.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.*;
import java.net.*;

public class FormPanel extends JPanel implements ListSelectionListener, ActionListener {

        // removied html formatting here because of problems on Windows...
	private final String NORMAL_FONT = "";          //"<html><font>";
	private final String HIGHLIGHTED_FONT = "->";   //"<html><font color=blue>";
	private final String END_TAGS = "";             //"</font></html>";
	
	PlayController controller;
	MetaMix parent;
	
	JPanel modePanel;
	JList list;
	JSplitPane splitPane;
	DefaultListModel sequences;
	int lastSelectedIndex = Traverser.DEFAULT_SEQUENCE;
	
	JButton setButton;
	JEditorPane info;
	JScrollPane listScrollPane, editorScrollPane;
	JLabel curForm;
    
	FormPanel (PlayController pc, MetaMix p) {
		
		controller = pc;
		parent = p;

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(Box.createRigidArea(new Dimension(0,10)));

		modePanel = new JPanel();
		modePanel.setLayout(new BoxLayout(modePanel, BoxLayout.X_AXIS));

		//Create the list of images and put it in a scroll pane
		sequences = new DefaultListModel();
		sequences.setSize(Traverser.NUM_SEQUENCES);
		for (int i=0; i < Traverser.NUM_SEQUENCES; i++) {
		    if (i == Traverser.DEFAULT_SEQUENCE) {
			sequences.set(i, HIGHLIGHTED_FONT + Traverser.getName(i) + END_TAGS);
			lastSelectedIndex = i;
		    } else
			sequences.set(i, NORMAL_FONT + Traverser.getName(i) + END_TAGS);
		}
		
		list = new JList(sequences);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.setFont(list.getFont().deriveFont(Font.PLAIN, 14));
		list.setFixedCellHeight(24);
		list.setSelectedIndex(0);
		list.addListSelectionListener(this);
		listScrollPane = new JScrollPane(list);
		modePanel.add(listScrollPane);
		modePanel.add(Box.createRigidArea(new Dimension(20,0)));
		
		JPanel setPanel = new JPanel();
		setPanel.setLayout(new BoxLayout(setPanel, BoxLayout.Y_AXIS));
		setPanel.add(Box.createVerticalGlue());
		JLabel setLabel = new JLabel("Current form:");
		setPanel.add(setLabel);
		setPanel.add(Box.createRigidArea(new Dimension(0,5)));
		curForm = new JLabel(Traverser.getName(Traverser.DEFAULT_SEQUENCE));
		setPanel.add(curForm);
		setPanel.add(Box.createRigidArea(new Dimension(0,20)));
		
		setButton = new JButton("change");
		setButton.addActionListener(this);
		setButton.setActionCommand("change");
		setPanel.add(setButton);
		setPanel.add(Box.createVerticalGlue());
		
		modePanel.add(setPanel);
		modePanel.add(Box.createHorizontalGlue());
		
		add(Box.createRigidArea(new Dimension(0,5)));
		add(Box.createVerticalGlue());
		
		try {
		    info = new JEditorPane(this.getClass().getResource("/html/" + Traverser.getDescription(lastSelectedIndex)));
		} catch (Exception e) {
		    try {
			info = new JEditorPane(this.getClass().getResource("/html/ANotFound.html"));
		    } catch (Exception e2) {
			info = new JEditorPane("text/plain", "");
		    }
		}
		info.setEditable(false);
		editorScrollPane = new JScrollPane(info);
		editorScrollPane.setVerticalScrollBarPolicy(
		    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		// now make the split pane
		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
		    modePanel, editorScrollPane);
		add(splitPane);
		setBorder(BorderFactory.createLoweredBevelBorder());
	}
	
	public void setDivider() {	
	    // set these after window is drawn so we know width and height
	    //splitPane.setDividerLocation((int)(this.getHeight() * 0.35));
	    // use parent's height to work around JRE 1.4.1 bug
	    int totalHeight = parent.getHeight();
	    splitPane.setDividerLocation((int)(totalHeight * 0.25));
	    // freeze scroll pane width so it doesn't change when mode labels change
	    listScrollPane.setMinimumSize(new Dimension(this.getWidth() / 2, modePanel.getMinimumSize().height));
	    listScrollPane.setMaximumSize(new Dimension(this.getWidth() / 2, modePanel.getMaximumSize().height));
	}
	
	public void valueChanged(ListSelectionEvent e) {
	    if (e.getValueIsAdjusting())
		return;

	    JList theList = (JList)e.getSource();
	    if (theList.isSelectionEmpty()) {
		;
	    } else {
		try {
		    info.setPage(this.getClass().getResource("/html/" + Traverser.getDescription(list.getSelectedIndex())));
		} catch (Exception e2) {
		    try {
			info.setPage(this.getClass().getResource("/html/ANotFound.html"));
		    } catch (Exception e3) {
			info.setText("");
		    }
		}
	    }
	}
	
	public void actionPerformed(ActionEvent e) {
	    if (e.getActionCommand().equals("change")) {
		sequences.set(lastSelectedIndex, NORMAL_FONT + Traverser.getName(lastSelectedIndex) + END_TAGS);
		int index = list.getSelectedIndex();
		lastSelectedIndex = index;
		sequences.set(index, HIGHLIGHTED_FONT + Traverser.getName(index) + END_TAGS);
		controller.setMode(index);
		parent.controls.setMode(index);
		curForm.setText(Traverser.getName(index));
	    }
	}
}
