/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * HelpBox.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class HelpBox extends JFrame implements ActionListener {

	private JEditorPane info;
	private JComboBox section;

	public HelpBox() {
	super("MetaMix Help");

	JLabel label;

	Container contentPane = getContentPane();
	contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
	contentPane.add(Box.createVerticalGlue());
	contentPane.add(Box.createRigidArea(new Dimension(0,10)));
	
	// create the section subpanel
	JPanel sectionPanel = new JPanel();
		sectionPanel.setLayout(new BoxLayout(sectionPanel, BoxLayout.X_AXIS));
		sectionPanel.add(Box.createHorizontalGlue());
	label = new JLabel("Go to:");
	sectionPanel.add(label);
	sectionPanel.add(Box.createRigidArea(new Dimension(5,0)));

	String[] sectionStrings = { "Quick Start", "Background", "How it Works", "User Controls", "Frequently Asked Questions"};
	section = new JComboBox(sectionStrings);
	section.setSelectedIndex(0);
	section.addActionListener(this);
	sectionPanel.add(section);
	sectionPanel.add(Box.createHorizontalGlue());
		contentPane.add(sectionPanel);

	contentPane.add(Box.createRigidArea(new Dimension(0,10)));
	
	try {
		info = new JEditorPane(this.getClass().getResource("/html/quickstart.html"));
	} catch (Exception e) {
		info = new JEditorPane("text/plain", "Error: Could not locate help files.");
	}
	info.setEditable(false);
	info.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	JScrollPane editorScrollPane = new JScrollPane(info);
	editorScrollPane.setVerticalScrollBarPolicy(
		JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	editorScrollPane.setMinimumSize(new Dimension(600,400));
	editorScrollPane.setPreferredSize(new Dimension(600,600));
	contentPane.add(editorScrollPane);
		
	contentPane.add(Box.createRigidArea(new Dimension(0,10)));
	contentPane.add(Box.createVerticalGlue());
	this.setResizable(true);
		pack();
	Dimension screenSize =
			Toolkit.getDefaultToolkit().getScreenSize();
	Dimension labelSize = contentPane.getPreferredSize();
	setLocation(screenSize.width - labelSize.width, 0);
	//setVisible(false);
	}
    
	public void showBox() {
	this.setVisible(true);
	}
    
	public void hideBox() {
	this.setVisible(false);
	}
    
	public void actionPerformed(ActionEvent e) {
	int selected = ((JComboBox)e.getSource()).getSelectedIndex();
	if (selected == 0) {
		setPage("/html/quickstart.html");
	} else if (selected == 1) {
		setPage("/html/background.html");
	} else if (selected == 2) {
		setPage("/html/howitworks.html");
	} else if (selected == 3) {
		setPage("/html/usercontrols.html");
	} else if (selected == 4) {
		setPage("/html/faq.html");
	}
	}
    
	private void setPage(String url) {
	try {
		info.setPage(this.getClass().getResource(url));
	} catch (Exception e) {
		info.setText("Error: Could not locate help files.");
	}
	}
}