/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * MetaMix.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class MetaMix extends JFrame {

	public static String VERSION = "1.1.2";
	public static boolean DEBUG = true;
	public static int MAX_NUMLAYERS = 6;
	public static int MAX_CHUNK_SIZE = 10;
	public static float DEFAULT_VOLUME = (float) 1.0;

	public ControlPanel controls;
	public JTabbedPane tabbedPane;
	public SourcePanel source;
	public MixPanel mix;
	public FormPanel form;
	public PlayController controller;
	public HelpBox helpBox;

	public MetaMix() {
		super("MetaMix");

		// check to see if QT for Java is here so we can exit gracefully if it's not
		try {
			Class.forName("quicktime.QTSession");
		} catch (Exception e) {
			pack();
			setVisible(true);
			if (!System.getProperty("os.name").startsWith("Mac")) {
				new Error(
					e,
					this,
					"Quicktime for Java not found; MetaMix must exit now.\nHit OK to open up a web page with information on how\nyou can fix this problem.");
				try {
					Runtime.getRuntime().exec(
						"cmd /c start http://www.jasonfreeman.net/MetaMix/win/check_qt.html");
				} catch (Exception e2) {
					try {
						Runtime.getRuntime().exec(
							"start http://www.jasonfreeman.net/MetaMix/win/check_qt.html");
					} catch (Exception e3) {
						new Error(
							e3,
							this,
							"Unable to launch web browser due to an unknown error.\n\nRemember that N.A.G. requires Windows 2000 or XP\nand DOES NOT SUPPORT Windows 95, 98 or ME.\n\nPlease visit:\n\nhttp://turbulence.org/Works/freeman/win/check_qt.html\n\nfor information on how you can install Quicktime for Java.");
					}
				}
			} else
				new Error(
					e,
					this,
					"Quicktime for Java not found\n"
						+ "If you are running Mac OS X 10.2.x with Quicktime 6.4,\n"
						+ "Please use Software Update to install the Quicktime for Java updater");
			System.exit(0);
		}


		if (!System.getProperty("os.name").startsWith("Mac")) {
			// set windows look and feel (aqua is already default for Mac OS X)
			try {
				UIManager.setLookAndFeel(
					"com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
				UIDefaults uid = UIManager.getLookAndFeelDefaults();
			} catch (Exception e) {
			}
		}

		controller = new PlayController(this);
		source = new SourcePanel(controller, this);
		mix = new MixPanel(controller, this);
		form = new FormPanel(controller, this);
		controls = new ControlPanel(controller, this);
		helpBox = new HelpBox();

		tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Source", source);
		tabbedPane.setSelectedIndex(0);
		tabbedPane.addTab("Form", form);
		tabbedPane.addTab("Mix", mix);

		Container contentPane = getContentPane();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		contentPane.add(tabbedPane);
		contentPane.add(controls);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				//System.out.println("ok...we're exiting normally now...");
				controller.close();
				System.exit(0);
			}
		});

		pack();

		int height = getHeight();
		int width = getWidth();
		if (height > 600)
			setSize(width, 600);
		if (width > 500)
			setSize(500, getHeight());

		// now, bring up open dialog immediately, before GUI even draws!!!
		// source.actionPerformed(new ActionEvent(this, 0, "import"));
		form.setDivider();
		// now that we've layed out and know the height of it...
	}

	private static SplashScreen splash;
	private static MetaMix main;

	public static void main(String args[]) {
		splash = new SplashScreen();
		main = new MetaMix();
		splash.setVisible(false);
		main.setVisible(true);
	}
}
