/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * MixPanel.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import javax.swing.*;
import javax.swing.JComponent.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.*;

public class MixPanel extends JPanel implements ItemListener, ChangeListener {

	PlayController controller;
	MetaMix parent;
	
	JSlider chunkSlider, layerSlider;
	JCheckBox reverseBox1, reverseBox2, counterBox;
	JLabel chunkSizeLabel;

	MixPanel (PlayController pc, MetaMix p) {
		
		controller = pc;
		parent = p;
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(Box.createVerticalGlue());
		
		JPanel chunkPanel = new JPanel();
		chunkPanel.setLayout(new BoxLayout(chunkPanel, BoxLayout.X_AXIS));
		chunkPanel.add(Box.createRigidArea(new Dimension(20,0)));
		
		JLabel label = new JLabel("Trigger new chunk every:", JLabel.LEFT);
		chunkPanel.add(label);
		chunkPanel.add(Box.createRigidArea(new Dimension(20,0)));
		
		/*label = new JLabel("short", JLabel.RIGHT);
		chunkPanel.add(label);*/		
		
		chunkSlider = new JSlider(10, 200, MetaMix.MAX_CHUNK_SIZE * 8);
		chunkSlider.setPaintTicks(false);
		chunkSlider.setPaintLabels(false);
		chunkSlider.setSnapToTicks(false);
		chunkSlider.addChangeListener(this);
		chunkPanel.add(chunkSlider);

		/*label = new JLabel("long", JLabel.LEFT);
		chunkPanel.add(label);	
		chunkPanel.add(Box.createRigidArea(new Dimension(20,0)));*/
		
		chunkSizeLabel = new JLabel(
			(String.valueOf(MetaMix.MAX_CHUNK_SIZE * 8 / (float) 20).substring(0,3) + " s"), JLabel.LEFT);
		chunkPanel.add(chunkSizeLabel);
		chunkPanel.add(Box.createHorizontalGlue());
		
		add(chunkPanel);
		add(Box.createRigidArea(new Dimension(0,30)));

		JPanel layerPanel = new JPanel();
		layerPanel.setLayout(new BoxLayout(layerPanel, BoxLayout.X_AXIS));
		layerPanel.add(Box.createRigidArea(new Dimension(20,0)));

		label = new JLabel("Simultaneous layers:", JLabel.LEFT);
		layerPanel.add(label);
		layerPanel.add(Box.createRigidArea(new Dimension(20,0)));

		layerSlider = new JSlider(1, MetaMix.MAX_NUMLAYERS, 2);
		layerSlider.setMajorTickSpacing(1);
		layerSlider.setPaintTicks(true);
		layerSlider.setPaintLabels(true);
		layerSlider.setSnapToTicks(true);		
		layerSlider.addChangeListener(this);
		layerPanel.add(layerSlider);		

		layerPanel.add(Box.createHorizontalGlue());
		add(layerPanel);
		add(Box.createRigidArea(new Dimension(0,30)));

		JPanel revlabel = new JPanel();
		revlabel.setLayout(new BoxLayout(revlabel, BoxLayout.X_AXIS));
		revlabel.add(Box.createRigidArea(new Dimension(20,0)));
		label = new JLabel("Play in reverse when:", JLabel.LEFT);
		revlabel.add(label);
		revlabel.add(Box.createHorizontalGlue());
		add(revlabel);
		
		add(Box.createRigidArea(new Dimension(0,10)));
		
		JPanel revbox1 = new JPanel();
		revbox1.setLayout(new BoxLayout(revbox1, BoxLayout.X_AXIS));
		revbox1.add(Box.createRigidArea(new Dimension(75,0)));
		reverseBox1 = new JCheckBox("integer is negative", false);
		reverseBox1.addItemListener (this);
		revbox1.add(reverseBox1);
		revbox1.add(Box.createHorizontalGlue());
		add(revbox1);

		JPanel revbox2 = new JPanel();
		revbox2.setLayout(new BoxLayout(revbox2, BoxLayout.X_AXIS));
		revbox2.add(Box.createRigidArea(new Dimension(75,0)));		
		reverseBox2 = new JCheckBox("next integer is smaller", false);
		reverseBox2.addItemListener (this);
		revbox2.add(reverseBox2);
		revbox2.add(Box.createHorizontalGlue());
		add(revbox2);
		
		add(Box.createRigidArea(new Dimension(0,30)));
		
		JPanel counter = new JPanel();
		counter.setLayout(new BoxLayout(counter, BoxLayout.X_AXIS));
		counter.add(Box.createRigidArea(new Dimension(20,0)));		
		counterBox = new JCheckBox("Slower counter animation (conserves CPU)", false);
		counterBox.addItemListener (this);
		counter.add(counterBox);
		counter.add(Box.createHorizontalGlue());
		add(counter);
		
		add(Box.createVerticalGlue());
		setBorder(BorderFactory.createLoweredBevelBorder());
	}

	public void itemStateChanged(ItemEvent e) {
	    Object source = e.getItemSelectable();
	    if (source == reverseBox1) {
		if (e.getStateChange() == ItemEvent.DESELECTED)
		    controller.setReverseNeg(false);
		else
		    controller.setReverseNeg(true);
	    } else if (source == reverseBox2) {
		if (e.getStateChange() == ItemEvent.DESELECTED)
		    controller.setReverseDec(false);
		else
		    controller.setReverseDec(true);
	    } else if (source == counterBox) {
		if (e.getStateChange() == ItemEvent.DESELECTED)
		    ((MetaMix) parent).controls.setTimerDelay(50);
		else
		    ((MetaMix) parent).controls.setTimerDelay(1000);
	    }
	}
		
	public void stateChanged(ChangeEvent event) {
	    JSlider s = (JSlider) event.getSource();
	    if ((s == layerSlider) && (!s.getValueIsAdjusting())) {
		int w = (int)s.getValue();
		controller.setNumPlayers(w);
	    } else if (s == chunkSlider) {
		int w = (int)s.getValue();
		chunkSizeLabel.setText(String.valueOf(w / (float) 20).substring(0,3) + " s");
		if (!s.getValueIsAdjusting()) {
		    controller.setChunkSize(w / (float) 20, controller.getOffset());
		}
	    }
	}
}
