/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * PlayController.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import quicktime.*;
import quicktime.app.time.TaskAllMovies;
import quicktime.app.time.TaskThread;
import quicktime.io.*;
import quicktime.sound.*;
import quicktime.std.*;
import quicktime.std.comp.*;
import quicktime.std.movies.*;
import quicktime.std.clocks.*;
import quicktime.std.qtcomponents.*;
import quicktime.std.movies.media.*;
import java.util.*;
import java.io.*;
import javax.swing.*;

public class PlayController {

	protected Vector playArray;
	protected int curPlayer = 0;
	protected SlavedPlayer curSlave;
	protected Traverser trav;
	protected boolean firstPlay = true;
	protected int numPlayers = 2;
	protected float chunkSize = 4;
	protected JFrame parentFrame;
	protected QTFile qtf = null;
	protected boolean reverseNeg = false;
	// play chunks backwards if negative integer
	protected boolean reverseDec = false;
	// play chunks backwards if next integer is lower
	private int fileNumber = 0; // hack to get around QTFile bug
	protected int numChunks = 40;
	protected float offset = 0; // where to start in soundfile (0 <= x < 1)
	protected float volume = MetaMix.DEFAULT_VOLUME;
	protected int mute = 1;
	protected int thisChunkNum = 0;
	// chunk number without absolute value (for display purposes)
	protected boolean showExportDialog = false;
	protected boolean triggerLocked = false;
	// our simple way of doing thread synchronization...

	public PlayController(JFrame f) {
		parentFrame = f;
		initPlayArray();
		trav = new Traverser(f);
		
		try {
			QTSession.open();
		} catch (QTException e1) {
			new Error(e1, parentFrame, "Unable to initialize Quicktime for an unknown reason");
		}

		if (QTSession.isCurrentOS(QTSession.kWin32)) {
			// give the tasker higher priority on windows machines
			TaskThread tt = TaskAllMovies.get().getDefaultTasker();
			tt.setPriority(Thread.MAX_PRIORITY);
			tt.setSleepTime(10);
			}
	}

	public void initPlayArray() {
		Vector newPlay = new Vector(MetaMix.MAX_NUMLAYERS + 1);
		for (int i = 0; i < (numPlayers + 1); i++) {
			if ((playArray != null) && (playArray.size() > i)) {
				try {
					((SlavedPlayer) playArray.elementAt(i)).finalize();
				} catch (QTException e) {
					; //ignore
				}
			}
			newPlay.addElement(new SlavedPlayer(this));
		}
		playArray = newPlay;
	}

	public boolean open(boolean convert) {
		QTFile oldqtf = qtf;
		String fileName;
		OpenMovieFile movieFile = null;
		Movie theMovie = null;

		parentFrame.setVisible(false);
		// so they don't muck around with controls or anything...
		if (playing() || paused())
			if (stop())
				 ((MetaMix) parentFrame).controls.stop();
		try {
			qtf = null;
			if (QTSession.isCurrentOS(QTSession.kWin32))
				// show all files since QTWin doesn't filter audio files well...
				qtf = QTFile.standardGetFilePreview(null);
			else
				qtf =
					QTFile.standardGetFilePreview(QTFile.kStandardQTFileTypes);
			// check to see if valid file was returned...
			try {
				movieFile = OpenMovieFile.asRead(qtf);
				theMovie = Movie.fromFile(movieFile);
				movieFile.close();
				theMovie.disposeQTObject();
			} catch (Exception e) {
				Error error =
					new Error(
						e,
						parentFrame,
						"MetaMix does not recognize this file. Please pick a CD Track or audio file (AIFF, WAV, MP3, ...)");
				qtf = oldqtf;
				if (movieFile != null)
					movieFile.close();
				if (theMovie != null)
					theMovie.disposeQTObject();
				parentFrame.setVisible(true);
				return (false);
			}
			fileName = qtf.getName();
			if (convert) {
				movieFile = OpenMovieFile.asRead(qtf);
				theMovie = Movie.fromFile(movieFile);
				qtf = convert(theMovie);
				try {
					movieFile.close();
					theMovie.disposeQTObject();
				} catch (Exception e) {
				}
			}
			initPlayArray();
			for (int i = 0; i < (numPlayers + 1); i++)
				 ((SlavedPlayer) playArray.elementAt(i)).open(qtf, numChunks);
		} catch (QTIOException e) {
			if (qtf == null) {
				// means user most likely hit "cancel" so don't bother showing anything...
				qtf = oldqtf;
				parentFrame.setVisible(true);
				try {
					movieFile.close();
					theMovie.disposeQTObject();
				} catch (Exception e2) {
				}
				return (false);
			}
			qtf = oldqtf;
			Error error =
				new Error(
					e,
					parentFrame,
					"Action canceled or unknown Quicktime IO error occurred.");
			parentFrame.setVisible(true);
			try {
				movieFile.close();
				theMovie.disposeQTObject();
			} catch (Exception e2) {
			}
			return (false);
		} catch (QTException e) {
			qtf = oldqtf;
			Error error =
				new Error(
					e,
					parentFrame,
					"Action canceled or unknown Quicktime error occurred.");
			parentFrame.setVisible(true);
			try {
				movieFile.close();
				theMovie.disposeQTObject();
			} catch (Exception e2) {
			}
			return (false);
		} catch (NullPointerException e) { // import dialog was canceled
			qtf = oldqtf;
			parentFrame.setVisible(true);
			try {
				movieFile.close();
				theMovie.disposeQTObject();
			} catch (Exception e2) {
			}
			return (false);
		}
		setOffset((int) (offset * 100));
		setChunkSize(chunkSize, offset);
		trav.reset();
		setPreRolls(true);
		((MetaMix) parentFrame).source.setFileName(fileName);
		try {
			oldqtf.delete();
		} catch (Exception e) {
		} // might as well delete early...
		parentFrame.setVisible(true);
		return (true);
	}

	public QTFile convert(Movie theMovie) throws QTException, StdQTException {
		QTFile file;

		try {
			file = new QTFile(File.createTempFile("MetaMix", ".aiff"));
			file.deleteOnExit();
		} catch (IOException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"An unknown Java filesystem error has occurred.");
			file = null;
			return (file);
		}

		ImportProgressBar progbar = new ImportProgressBar(parentFrame);

		MovieExporter theMovieExp = null;
		// we have to do this the long way because of a strange problem in
		// QT where it sometimes tries to use a MIDI->AIFF exporter instead...
		ComponentDescription cd =
			new ComponentDescription(
				StdQTConstants.movieExportType,
				StdQTConstants.kQTFileTypeAIFF,
				StdQTConstants.soundMediaType,
				0,
				0);
		ComponentIdentifier ci = ComponentIdentifier.find(cd);
		//System.out.println(cd.count() + " entries match component search criteria.");
		theMovieExp = new MovieExporter(ci);

		//theMovieExp = new MovieExporter(StdQTConstants.kQTFileTypeMovie);

		if (showExportDialog)
			if (theMovieExp
				.doUserDialog(theMovie, null, 0, theMovie.getDuration())) {
				file = null;
				return (file); // dialog was cancelled
			}

		Win32Progress winprog = null;

		//with new QTJ this problem now happens on OS X too!
		//if (QTSession.isCurrentOS(QTSession.kWin32)) {
		// because of stupid problems with progress bar on Windows...
		winprog = new Win32Progress();
		try { Thread.sleep(1000); } catch (Exception e) { } // because QTJ ON WINDOWS SUCKS!!!!!!!!!!!1
		//} else
		//theMovieExp.setProgressProc(progbar);

		// this is commented out because it seems to cause problems on OSX
		/*file.createMovieFile (StdQTConstants.kMoviePlayer,
				StdQTConstants.createMovieFileDontCreateResFile | 
				StdQTConstants.createMovieFileDeleteCurFile);*/

		theMovieExp.toFile(
			file,
			theMovie,
			null,
			0,
			theMovie.getDuration());
		//if (QTSession.isCurrentOS(QTSession.kWin32))
		winprog.setVisible(false);
		/*else
		if (progbar.isCanceled()) {
			file = null;
			return (file);
			}*/
		return (file);
	}

	public void setNumChunks(int w, float oldoffset, float oldChunkSize) {
		numChunks = w;
		try {
			boolean isplaying = playing();
			pause();
			trav.setSize(w);
			for (int i = 0; i < (numPlayers + 1); i++) {
				if (i == curPlayer)
					((SlavedPlayer) playArray.elementAt(i)).setNumChunks(
						w,
						true,
						oldoffset,
						oldChunkSize);
				else
					((SlavedPlayer) playArray.elementAt(i)).setNumChunks(
						w,
						false,
						oldoffset,
						oldChunkSize);
			}
			if (isplaying)
				play();
		} catch (StdQTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"An unknown Quicktime error occurred while trying to update chunk size.");
		} catch (QTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"An unknown Quicktime error occurred while trying to update chunk size.");
		}
	}

	public void setChunkSize(float s, float oldoffset) {
		float oldChunkSize = chunkSize;
		chunkSize = s;
		SlavedPlayer p = (SlavedPlayer) playArray.elementAt(0);
		try {
			for (int i = 0; i < (numPlayers + 1); i++) {
				SlavedPlayer slave = (SlavedPlayer) playArray.elementAt(i);
				slave.setChunkSize(s);
			}
			// we subtract here the 10 secs of silence buffer at the end...
			if (s == 0)
				setNumChunks(1, oldoffset, oldChunkSize);
			else
				setNumChunks(
					(int) ((p.getDuration() - (10 * p.getScale()))
						* (1.0 - offset)
						/ (s * p.getScale())),
					oldoffset,
					oldChunkSize);
		} catch (StdQTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"An unknown Quicktime error occurred while trying to update chunk size.");
		} catch (NullPointerException e) {
			// ignore
		}
	}

	public void setOffset(int n) {
		//System.out.println("setting offset");
		float oldoffset = offset;
		offset = (float) (n / 100.0);
		try {
			boolean isplaying = playing();
			pause();
			for (int i = 0; i < (numPlayers + 1); i++)
				 ((SlavedPlayer) playArray.elementAt(i)).setOffset(offset);
			setChunkSize(chunkSize, oldoffset);
			//System.out.println("restarting...");
			if (isplaying)
				play();
			//System.out.println("done...");
		} catch (StdQTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"An unknown Quicktime error occurred while trying to update offset.");
		} catch (QTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"An unknown Quicktime error occurred while trying to update offset.");
		}
	}

	public float getOffset() {
		return (offset);
	}

	public float getDuration() {
		try {
			SlavedPlayer slave = (SlavedPlayer) playArray.elementAt(0);
			if ((slave != null) && (slave.getDuration() != 0))
				return (slave.getDuration() / slave.getScale());
			return (0);
		} catch (Exception e) {
			return (0);
		}
	}

	private int findoverlap() throws StdQTException, QTException {
		// check for overlap with active player
		SlavedPlayer slave = null;
		for (int i = 0; i < (numPlayers + 1); i++) {
			if (i != curPlayer) {
				int differential;
				if (i < curPlayer)
					differential = curPlayer - i;
				else
					differential = curPlayer + (numPlayers + 1 - i);
				if (curSlave.getReverse())
					differential = -differential;
				slave = (SlavedPlayer) playArray.elementAt(i);
				if (slave.playing()
					&& (slave.getCurChunk()
						== (curSlave.getCurChunk() - differential))
					&& (slave.getReverse() == curSlave.getReverse())) {
					return (i);
				}
			}
		}
		return (-1); // no overlap
	}

	private void swapPlayers(int a, int b) {
		// swap players a and b
		Object temp = playArray.elementAt(a);
		playArray.setElementAt(playArray.elementAt(b), a);
		playArray.setElementAt(temp, b);
	}

	public void trigger() {
		//System.out.println("can I trigger?");
		triggerLocked = true;
		//System.out.println("starting to trigger...");
		try {
			int overlap = findoverlap();
			if ((overlap == -1) || (numPlayers == 1)) {
				// no need to swap when only 1 player active...
				 ((MetaMix) parentFrame).controls.setChunkInfo(curSlave);
				curSlave.play();
				//System.out.println("triggering player #" + curPlayer);
			} else { // there's an overlap
				//System.out.println("swapping players " + overlap + " and " + curPlayer);
				swapPlayers(overlap, curPlayer);
				SlavedPlayer slave =
					(SlavedPlayer) playArray.elementAt(overlap);
				slave.stop();
				slave.cancelTimer();
				slave = (SlavedPlayer) playArray.elementAt(curPlayer);
				slave.update(curSlave.getCurChunk(), true);
				//System.out.println("startTime " + slave.getStartTime() + " time " + slave.getTime() + " trigger " + slave.getTrigger());
				 ((MetaMix) parentFrame).controls.setChunkInfo(slave);
				//System.out.println("finished the swap..");
			}
			firstPlay = false;
			curPlayer = (curPlayer + 1) % (numPlayers + 1);
			//System.out.println("curPlayer incremented to " + curPlayer);
			setPreRolls(true);
			/* System.out.print("active players are ");
			for (int i=0; i<numPlayers+1; i++) {
			if (((SlavedPlayer)playArray.elementAt(i)).playing())
			    System.out.print(" " + i + " at time " + ((SlavedPlayer)playArray.elementAt(i)).getTime() + " ");
			}
			//System.out.println(); */
		} catch (Exception e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"An unknown Quicktime Error occurred during playback.");
			triggerLocked = false;
		}
		triggerLocked = false;
		//System.out.println("done triggering!");
	}

	public boolean play() {
		/*if (qtf == null) {
		    Error error = new Error(e, parentFrame, "You must first pick a source file (under the source tab).");
		    return(false);
		} */ // this code was redundant...
		try {
			qtf.toString();
			// derefence qtf to trigger NullPointerException before we mess around with anything...
			if (firstPlay) {
				trigger();
				firstPlay = false;
			} else
				for (int i = 0; i < (numPlayers + 1); i++) {
					SlavedPlayer slave =
						((SlavedPlayer) playArray.elementAt(i));
					if (slave.paused())
						slave.play();
				}
		} catch (QTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occured while trying to start playback.");
			trav.reset();
			curPlayer = 0;
			setPreRolls(true);
			firstPlay = true;
			return (false);
		} catch (NullPointerException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"You must first pick a source file (under the source tab).");
			trav.reset();
			curPlayer = 0;
			setPreRolls(true);
			firstPlay = true;
			return (false);
		}
		return (true);
	}

	public boolean pause() {
		if (qtf == null)
			return (false);
		try {
			for (int i = 0; i < (numPlayers + 1); i++)
				 ((SlavedPlayer) playArray.elementAt(i)).pause();
		} catch (StdQTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occured while trying to pause playback");
			return (false);
		} catch (QTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occured while trying to pause playback");
			return (false);
		}
		return (true);
	}

	public boolean stop() {
		if (!pause())
			return (false);
		try {
			for (int i = 0; i < (numPlayers + 1); i++)
				 ((SlavedPlayer) playArray.elementAt(i)).reinit();
			curPlayer = 0;
			trav.reset();
			setPreRolls(true);
			firstPlay = true;
		} catch (StdQTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occurred while trying to stop playback.");
			return (false);
		} catch (QTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occurred while trying to stop playback.");
			return (false);
		} catch (NullPointerException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"You must first pick a source file (under the source tab).");
			return (false);
		}
		return (true);
	}

	public boolean playing() {
		try {
			for (int i = 0; i < (numPlayers + 1); i++)
				if (((SlavedPlayer) playArray.elementAt(i)).playing())
					return (true);
			return (false);
		} catch (StdQTException e) {
			return (false);
		} catch (QTException e) {
			return (false);
		}
	}

	public boolean paused() {
		try {
			for (int i = 0; i < (numPlayers + 1); i++)
				if (((SlavedPlayer) playArray.elementAt(i)).paused())
					return (true);
			return (false);
		} catch (StdQTException e) {
			return (false);
		} catch (QTException e) {
			return (false);
		}
	}

	public void skip() {
		trav.skipAhead();
		setPreRolls(true);
	}

	public void back() {
		trav.goBack();
		setPreRolls(true);
	}

	// if advance is true, then we advance one integer in the sequence
	// else, we just update things for the current integer
	public void setPreRolls(boolean advance) {
		try {
			if (advance)
				thisChunkNum = trav.next();
			else
				thisChunkNum = trav.current();
			curSlave = (SlavedPlayer) playArray.elementAt(curPlayer);
			boolean r =
				((reverseNeg) && (thisChunkNum < 0))
					|| ((reverseDec)
						&& (Math.abs(thisChunkNum) > Math.abs(trav.peekAhead())));
			curSlave.preroll(Math.abs(thisChunkNum), r);
			//System.out.println("pre-rolled player " + curPlayer + " this val " + thisChunkNum + " next val " + trav.peekAhead() + " reverse? " + r +  " time " + curSlave.getTime());
		} catch (StdQTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occurred while trying to continue playback.");
		} catch (QTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occurred while trying to continue playback.");
		}
	}

	public void setReverseNeg(boolean r) {
		reverseNeg = r;
		setPreRolls(false);
	}

	public void setReverseDec(boolean r) {
		reverseDec = r;
		setPreRolls(false);
	}

	public void setMode(int m) {
		trav.setMode(m);
		trav.reset();
		setPreRolls(true);
	}

	public void setNumPlayers(int n) {
		boolean isPlaying = playing();
		pause();
		try {
			//System.out.println("curPlayer is " + curPlayer);
			Vector newPlay = new Vector(MetaMix.MAX_NUMLAYERS + 1);
			boolean increasing = (n > numPlayers);

			// copy the current players we still need, from youngest to oldest
			int playersToCopy;
			// don't add extra 1 so we can add curPlayer at end...
			if (!increasing)
				playersToCopy = n;
			else
				playersToCopy = numPlayers;
			for (int i = 0; i < playersToCopy; i++) {
				int index =
					(((curPlayer - i - 1) + (numPlayers + 1))
						% (numPlayers + 1));
				//System.out.println("copied player at " + index);
				SlavedPlayer slave = (SlavedPlayer) playArray.elementAt(index);
				//System.out.println("startTime " + slave.getStartTime() + " time " + slave.getTime() + " trigger " + slave.getTrigger());
				newPlay.add(0, playArray.elementAt(index));
			}

			// add curPlayer to the end
			newPlay.add(playArray.elementAt(curPlayer));

			// add extra players we need to end
			int playersToAdd = n - numPlayers;
			for (int i = 0; i < playersToAdd; i++) {
				SlavedPlayer slave = new SlavedPlayer(this);
				newPlay.addElement(slave);
				//System.out.println("added player");
				if (qtf != null)
					slave.open(qtf);
				slave.setChunkSize(chunkSize);
				//System.out.println("new player is paused? " + slave.paused());	
			}

			// kill unneeded players from playArray
			for (int i = playersToCopy;
				i < numPlayers;
				i++) { // don't delete curPlayer!!!
				int index =
					(((curPlayer - i - 1) + (numPlayers + 1))
						% (numPlayers + 1));
				//System.out.println("killed player at index " + index);
				SlavedPlayer slave = (SlavedPlayer) playArray.elementAt(index);
				slave.finalize();
			}

			// reset curPlayer and prerolls and cleanup
			curPlayer = n;
			if (n > numPlayers)
				curPlayer = numPlayers;
			//System.out.println("curPlayer is " + curPlayer);
			playArray = newPlay;
			setPreRolls(false);
			numPlayers = n;

		} catch (QTIOException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occurred while trying to update layering.");
		} catch (QTException e) {
			Error error =
				new Error(
					e,
					parentFrame,
					"A Quicktime error occurred while trying to update layering.");
		}
		if (isPlaying)
			play();
	}

	public void reset() throws StdQTException, QTException {
		curPlayer = 0;
		for (int i = 0; i < (numPlayers + 1); i++)
			 ((SlavedPlayer) playArray.elementAt(i)).reinit();
		firstPlay = true;
		setPreRolls(true);
	}

	public int getNumPlayers() {
		return (numPlayers);
	}

	public SlavedPlayer getPlayerAt(int i) {
		return ((SlavedPlayer) playArray.elementAt(i));
	}

	private void updateVolumes() {
		for (int i = 0; i < (numPlayers + 1); i++)
			 ((SlavedPlayer) playArray.elementAt(i)).setVolume(volume * mute);
	}

	public void setVolume(int v) {
		volume = v / (float) 100;
		updateVolumes();
	}

	public void setMute(int m) {
		mute = m;
		updateVolumes();
	}

	// naming import/export is weird, b/c app pretends it's importing it when it's actually exporting to tmp file
	public void setImportDialog(boolean s) {
		showExportDialog = s;
	}

	public boolean triggerLocked() {
		return (triggerLocked);
	}

	/**
	 * 
	 */
	protected void close() {
		QTSession.close();
	}

}
