/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * SlavedPlayer.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* NOTE: TaskAllMovies and loadIntoRam stuff is only for Win32 since it seems to cause more
	problems than it solves on MAC OSX. */

import quicktime.*;
import quicktime.app.time.*;
import quicktime.io.*;
import quicktime.std.*;
import quicktime.std.clocks.*;
import quicktime.std.movies.*;
import java.awt.event.*;

public class SlavedPlayer implements ActionListener {

	private Movie m;
	private javax.swing.Timer timer;
	private QTTimer qtTimer;
	private int offset = 0;
	private int chunkSize = 1;
	private int startTime = 0;
	private int curChunk = 0;
	private int originalChunk = 0; // chunk before overlap corrections, etc.
	private float volume = MetaMix.DEFAULT_VOLUME;
	private float theVolume = 0; // actual volume, not user-controlled level
	private boolean reverse = false;
	private boolean paused = false;
	private boolean updated = false;
	// update() was called to extend active player
	private PlayController parent;

	// SlavedPlayer is meant to handle one file over its lifetime.
	// To open a new file, finalize this object and instantiate a new one
	// (no big deal, since you'd be resetting everything anyways...
	public SlavedPlayer(PlayController p) {
		parent = p;
	}

	public void open(QTFile qtf) throws QTIOException, QTException {
		OpenMovieFile movieFile = OpenMovieFile.asRead(qtf);
		open(movieFile);
	}

	public void open(OpenMovieFile movieFile)
		throws QTIOException, QTException {
		m = Movie.fromFile(movieFile);
		timer = new javax.swing.Timer(25, this);
		timer.setInitialDelay(0);
		timer.setCoalesce(true);
		qtTimer = new QTTimer(this);
		m.setVolume(0);
		theVolume = 0;
	}

	public void open(QTFile qtf, int numChunks)
		throws QTIOException, QTException {
		open(qtf);
		chunkSize = (m.getDuration() - offset) / numChunks;
	}

	public void play() throws StdQTException, QTException {
		if (m != null) {
			int rate = 1;
			if (reverse)
				rate = -1;
			//System.out.println("startTime is " + startTime + " curTime is " + m.getTime() + " trigger at " + qtTimer.getValue());
			timeChanged(m.getTime());
			if (timer != null)
				timer.start();
			m.setRate(rate);
			if (QTSession.isCurrentOS(QTSession.kWin32))
				TaskAllMovies.addMovieAndStart();
			//System.out.println("tasking " + TaskAllMovies.currentMovieCount() + " movies");
		}
	}

	public void stop() throws StdQTException, QTException {
		if (m != null) {
			if (playing()) {
				//System.out.println("starting to stop...");
				paused = false;
				timer.stop();
				m.setRate(0);
				theVolume = 0;
				m.setVolume(0);
				if (QTSession.isCurrentOS(QTSession.kWin32))
					TaskAllMovies.removeMovie();
				//System.out.println("stopping stopping...");
			}
		}
	}

	public void pause() throws StdQTException, QTException {
		if (m != null) {
			if (playing()) {
				paused = true;
				timer.stop();
				m.setRate(0);
				m.setVolume(0);
				if (QTSession.isCurrentOS(QTSession.kWin32))
					TaskAllMovies.removeMovie();
			}
		}
	}

	public void jumpToChunk(int chunkNumber, boolean r)
		throws StdQTException, QTException {
		curChunk = chunkNumber;
		originalChunk = curChunk;
		reverse = r;
		if (m != null) {
			pause();
			startTime = chunkNumber * chunkSize + offset;
			m.setTimeValue(startTime);
			//updateTimer();
			play();
		}
	}

	// extends play of active slave as if it had actually started at new chunkNumber
	public void update(int chunkNumber, boolean checkvolume)
		throws StdQTException, QTException {
		if (timer != null)
			timer.stop();
		curChunk = chunkNumber;
		updated = checkvolume;
		startTime = chunkNumber * chunkSize + offset;
		/* if ((m.getTime() < startTime) && checkvolume) {
		    System.out.println("time is: " + m.getTime() + " but startime was set to " + startTime + " curChunk is " + curChunk + 
			" orig chunk " + getOriginalChunk());
		} */
		if (QTSession.isCurrentOS(QTSession.kWin32))
			cacheIntoRam(startTime, chunkSize, reverse, false);
		if (qtTimer.isSet()
			|| checkvolume) // i.e. when updating from user params don't set bogus timers!!!
			updateTimer();
		if (timer != null)
			timer.start();
	}

	public void preroll(int chunkNumber, boolean r)
		throws StdQTException, QTException {
		if (timer != null)
			timer.stop();
		curChunk = chunkNumber;
		originalChunk = curChunk;
		reverse = r;
		int rate = 1;
		if (reverse)
			rate = -1;
		if (m != null) {
			stop();
			startTime = chunkNumber * chunkSize + offset;
			if (QTSession.isCurrentOS(QTSession.kWin32))
				cacheIntoRam(startTime, chunkSize, reverse, true);
			m.setTimeValue(startTime);
			m.prePreroll(startTime, rate);
			m.preroll(startTime, rate);
			updateTimer();
			//System.out.println("prerolled player to " + m.getTime() + " startTime " + startTime);
		}
	}

	public void cacheIntoRam(
		final int startTime,
		final int chunkSize,
		final boolean reverse,
		final boolean purgeFirst) {
		// we do this in a separate thread so it doesn't screw anything up...
		Thread t = new Thread() {
			public void run() {
				try {
					if (purgeFirst) { // we don't want to purge if this is called from update()
						// System.out.println("purging...");
						m.loadIntoRam(
							0,
							m.getDuration(),
							StdQTConstants.flushFromRam);
						// System.out.println("purging done");
					}
					// System.out.println("caching...");
					if (reverse)
						m.loadIntoRam(
							startTime
								- (chunkSize * (parent.getNumPlayers() + 1)),
							chunkSize * parent.getNumPlayers(),
							StdQTConstants.keepInRam);
					else
						m.loadIntoRam(
							startTime,
							chunkSize * (parent.getNumPlayers() + 1),
							StdQTConstants.keepInRam);
					// System.out.println("caching done");
				} catch (Exception e) {
					// e.printStackTrace();
					// ignore it...
				}
			}
		};
		t.start();
	}

	public void cancelTimer() {
		qtTimer.cancel();
	}

	private void updateTimer() throws StdQTException, QTException {
		int newTrigger;

		if (!reverse) {
			if ((startTime + getChunkSize()) > getDuration())
				// this should never happen!!!
				newTrigger = getDuration() - 1;
			else
				newTrigger = startTime + getChunkSize();
		} else {
			if ((startTime - getChunkSize()) < offset)
				// this should never happen!!!
				newTrigger = offset;
			else
				newTrigger = startTime - getChunkSize();
		}
		qtTimer.reset(newTrigger);
	}

	public int getStartTime() {
		return (startTime);
	}

	public int getTrigger() {
		if (qtTimer != null)
			return (qtTimer.getValue());
		return (-1);
	}

	public int getTime() throws StdQTException {
		if (m != null)
			return (m.getTime());
		return (-1);
	}

	public int getCurChunk() {
		return (curChunk);
	}

	public int getOriginalChunk() {
		return (originalChunk);
	}

	public void setNumChunks(
		int c,
		boolean current,
		float oldoffset,
		float oldChunkSize)
		throws StdQTException, QTException {
		int oldCS = (int) (oldChunkSize * getScale());
		int oldO = (int) (oldoffset * getDuration());
		//System.out.println("time was at: " + m.getTime() + " next trigger at " + qtTimer.getValue());
		if (m != null) {
			double percentdone =
				(m.getTime() - (oldCS * curChunk + oldO)) / (double) oldCS;
			if (c == 0)
				curChunk = 0;
			else if (curChunk > c)
				curChunk = curChunk % c;
			m.setTimeValue(
				(int) (curChunk * chunkSize
					+ offset
					+ chunkSize * percentdone));
			update(curChunk, false);
			int rate = 1;
			if (reverse)
				rate = -1;
			m.prePreroll(m.getTime(), rate);
			m.preroll(m.getTime(), rate);
			//System.out.println("player updated to time " + m.getTime() + " startTime " + startTime + " trigger " + qtTimer.getValue());	     
		}
	}

	// percdone till next trigger of a player; not till this one is finished!!!
	public float getPercDone() throws StdQTException {
		if ((m == null) || (chunkSize == 0))
			return (0);
		float result =
			Math.abs((m.getTime() - startTime) / ((float) chunkSize));

		while (result > 1) {
			//System.out.println("result is: " + result);
			result = result - 1;
		}
		return (result);
	}

	public int getChunkSize() {
		return (chunkSize);
	}

	public void setChunkSize(float s) throws StdQTException {
		// after calling this, you must also update with setNumChunks...
		if (m != null) {
			chunkSize = (int) (s * getScale());
		}
	}

	public void setOffset(float o) throws StdQTException, QTException {
		// after calling this, you must also update with setNumChunks...
		if (m != null)
			offset = (int) (o * getDuration());
	}

	public void setVolume(float v) {
		volume = v;
	}

	// real volume, not user setting
	public float getActualVolume() {
		return (theVolume / volume);
	}

	public boolean playing() throws StdQTException, QTException {
		if ((m != null) && (m.getRate() != 0))
			return true;
		return false;
	}

	public boolean paused() throws StdQTException, QTException {
		return (paused);
	}

	public TimeBase getTimeBase() throws StdQTException, NullPointerException {
		return (m.getTimeBase());
	}

	public int getScale() throws StdQTException, NullPointerException {
		return (m.getTimeScale());
	}

	public int getDuration() throws StdQTException, NullPointerException {
		if (m != null)
			return (m.getDuration());
		else
			return (0);
	}

	public boolean getReverse() {
		return (reverse);
	}

	public void reinit() throws StdQTException, QTException {
		paused = false;
		if (m != null) {
			m.setVolume(0);
			theVolume = 0;
			startTime = 0;
			m.setTimeValue(0);
		}
	}

	public void timeChanged(int thisTime) throws QTException {
		int timeElapsed;
		float multiplier = 1;

		timeElapsed = Math.abs(thisTime - startTime);
		if ((getDuration() - thisTime < m.getTimeScale()))
			multiplier =
				(getDuration() - thisTime + 1) / (float) m.getTimeScale();
		float newVolume =
			(float) Math.sin(
				Math.PI
					* timeElapsed
					/ (float) (getChunkSize() * parent.getNumPlayers()))
				* multiplier;
		if (!updated
			|| (Math.abs(newVolume * volume - theVolume) < 0.01)) {
			// to make smooth transitions on updates..
			theVolume = newVolume * volume;
			updated = false;
		} else if (volume == 0) { // just catch a mute!
			theVolume = 0;
		} else if (theVolume > (volume * newVolume))
			theVolume -= 0.01;
		else
			theVolume += 0.01;
		m.setVolume(theVolume);
		qtTimer.checkAndExecute(thisTime, reverse);
		// System.out.println("thistime " + thisTime + " start " + startTime + " timeElapsed " + timeElapsed + " chunk " + getChunkSize());
	}

	public void actionPerformed(ActionEvent e) {
		try {
			timeChanged(m.getTime());
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public void execute() {
		if (!parent.triggerLocked()) {
			qtTimer.cancel();
			parent.trigger();
		}
	}

	public void finalize() throws QTException {
		if (timer != null) {
			timer.stop();
		}
		if (m != null) {
			pause();
			m.disposeQTObject();
		}
	}
}
