/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * SourcePanel.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import javax.swing.*;
import javax.swing.JComponent.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.*;
import quicktime.std.*;

public class SourcePanel
	extends JPanel
	implements ActionListener, ChangeListener, ItemListener {

	private PlayController controller;
	private MetaMix parent;

	private JButton importButton, openButton;
	private JSlider startSlider;
	private JRadioButton goodSound, greatSound;
	private JLabel startTime, fileLabel;
	private JCheckBox importDialog;

	SourcePanel(PlayController pc, MetaMix p) {

		controller = pc;
		parent = p;

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		JPanel basicPanel = new JPanel();
		basicPanel.setLayout(new BoxLayout(basicPanel, BoxLayout.Y_AXIS));
		basicPanel.add(Box.createVerticalGlue());

		JPanel filePanel = new JPanel();
		filePanel.setLayout(new BoxLayout(filePanel, BoxLayout.X_AXIS));
		filePanel.add(Box.createRigidArea(new Dimension(20, 0)));

		JLabel label =
			new JLabel("Pick audio file (MP3, WAV, AIFF, ...):", JLabel.LEFT);
		filePanel.add(label);

		/*openButton = new JButton ("Import in place..."); // for debugging only
		openButton.setActionCommand("openinplace");
		openButton.addActionListener(this);
		filePanel.add(openButton);*/

		filePanel.add(Box.createRigidArea(new Dimension(20, 0)));

		importButton = new JButton("Pick source...");
		importButton.setActionCommand("import");
		importButton.addActionListener(this);
		filePanel.add(importButton);
		filePanel.add(Box.createRigidArea(new Dimension(10, 0)));
		filePanel.add(Box.createHorizontalGlue());

		basicPanel.add(filePanel);

		basicPanel.add(Box.createRigidArea(new Dimension(0, 5)));

		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		infoPanel.add(Box.createRigidArea(new Dimension(20, 0)));

		label = new JLabel("Current File: ", JLabel.LEFT);
		infoPanel.add(label);
		fileLabel = new JLabel("(None)", JLabel.LEFT);
		infoPanel.add(fileLabel);
		infoPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		infoPanel.add(Box.createHorizontalGlue());

		basicPanel.add(infoPanel);

		basicPanel.add(Box.createRigidArea(new Dimension(0, 70)));

		JPanel startPanel = new JPanel();
		startPanel.setLayout(new BoxLayout(startPanel, BoxLayout.X_AXIS));
		startPanel.add(Box.createRigidArea(new Dimension(20, 0)));

		label = new JLabel("Start from:", JLabel.LEFT);
		startPanel.add(label);
		startPanel.add(Box.createRigidArea(new Dimension(20, 0)));

		label = new JLabel("beginning", JLabel.RIGHT);
		startPanel.add(label);

		startSlider = new JSlider(0, 95, 0);
		startSlider.setPaintTicks(false);
		startSlider.setPaintLabels(true);
		startSlider.setSnapToTicks(false);
		startSlider.addChangeListener(this);
		startPanel.add(startSlider);

		label = new JLabel("end", JLabel.LEFT);
		startPanel.add(label);
		startPanel.add(Box.createRigidArea(new Dimension(20, 0)));

		startTime = new JLabel("0:00", JLabel.RIGHT);
		startPanel.add(startTime);
		startPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		startPanel.add(Box.createHorizontalGlue());

		basicPanel.add(startPanel);

		basicPanel.add(Box.createVerticalGlue());

		JPanel advancedPanel = new JPanel();
		advancedPanel.setLayout(new BoxLayout(advancedPanel, BoxLayout.X_AXIS));
		advancedPanel.add(Box.createRigidArea(new Dimension(20, 0)));
		importDialog =
			new JCheckBox(
				"show import audio settings dialog (advanced users only)",
				false);
		importDialog.addItemListener(this);
		advancedPanel.add(importDialog);
		advancedPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		advancedPanel.add(Box.createHorizontalGlue());

		advancedPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createLoweredBevelBorder(),
				BorderFactory.createEmptyBorder(30, 0, 30, 0)));
		basicPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createLoweredBevelBorder(),
				BorderFactory.createEmptyBorder(20, 0, 0, 0)));
		add(basicPanel);
		add(advancedPanel);
	}

	public void updateStartTime(int seconds) {
		String text = (seconds / 60) + ":";
		if ((seconds % 60) < 10)
			text = text + "0";
		text = text + (seconds % 60);
		startTime.setText(text);
	}

	public void setFileName(String s) {
		fileLabel.setText(s);
		parent.controls.setFile(s);
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("import")) {
			Thread t = new Thread() {
				public void run() {
					if (controller.open(true))
						updateStartTime(
							(int) (controller.getDuration()
								* controller.getOffset()));
				}
			};
			t.start();
		} else if (e.getActionCommand().equals("openinplace")) {
			controller.open(false);
			updateStartTime(
				(int) (controller.getDuration() * controller.getOffset()));
		}
	}

	public void stateChanged(ChangeEvent event) {
		JSlider s = (JSlider) event.getSource();
		if (s == startSlider) {
			int w = (int) s.getValue();
			updateStartTime(
				(int) (controller.getDuration() * (w / (float) 100)));
		}
		if ((s == startSlider) && (!s.getValueIsAdjusting())) {
			int w = (int) s.getValue();
			controller.setOffset(w);
		}
	}

	public void itemStateChanged(ItemEvent e) {
		Object source = e.getItemSelectable();
		if (source == importDialog) {
			if (e.getStateChange() == ItemEvent.DESELECTED)
				controller.setImportDialog(false);
			else
				controller.setImportDialog(true);
		}
	}

}
