/*
 * MetaMix
 *
 * copyright (c) 2001-2004 by Jason Freeman
 *
 * Traverser.java
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.util.*;
import javax.swing.*;

public class Traverser {

	// traversal modes
	// reference numbers are from On-Line Encylopedia of Integer Sequences
	// http://www.research.att.com/~njas/sequences
	public static final int _A000120 = 0;
	public static final int _A023416 = 1;
	public static final int _A062756 = 2;
	public static final int _A005590 = 3;
	public static final int _A002487 = 4;
	public static final int _A005812 = 5;
	public static final int _A037834 = 6;
	public static final int _A053735 = 7;
	public static final int _A006287 = 8;
	public static final int _A008687 = 9;
	public static final int _A038760 = 10;
	public static final int _A051734 = 11;
	public static final int _ARAND = 12;
	public static final int NUM_SEQUENCES = 13;
	public static final int DEFAULT_SEQUENCE = _A000120;

	private long index = 0;
	private int size = 40;
	private Sequence sequence;
	private int mode = DEFAULT_SEQUENCE;

	private int currentVal = -1;
	private int nextVal;
	private boolean computedNext = false;

	private JFrame parentFrame;

	public Traverser(int size) {
		sequence = new A000120();
		setSize(size);
	}

	public Traverser(JFrame f) {
		sequence = new A000120();
		setSize(size);
		parentFrame = f;
	}

	public void setSize(int s) {
		size = s;
		sequence.setSize(s);
	}

	public int getSize() {
		return (size);
	}

	public void setMode(int m) {
		computedNext = false;
		mode = m;
		switch (m) {
			case _ARAND :
				sequence = new ARand();
				break;
			case _A000120 :
				sequence = new A000120();
				break;
			case _A023416 :
				sequence = new A023416();
				break;
			case _A062756 :
				sequence = new A062756();
				break;
			case _A005590 :
				sequence = new A005590();
				break;
			case _A005812 :
				sequence = new A005812();
				break;
			case _A006287 :
				sequence = new A006287();
				break;
			case _A008687 :
				sequence = new A008687();
				break;
			case _A037834 :
				sequence = new A037834();
				break;
			case _A053735 :
				sequence = new A053735();
				break;
			case _A002487 :
				sequence = new A002487();
				break;
			case _A038760 :
				sequence = new A038760();
				break;
			case _A051734 :
				sequence = new A051734();
				break;
		}
		sequence.setSize(size);
	}

	public static String getName(int index) {
		switch (index) {
			case _ARAND :
				return ARand.name;
			case _A000120 :
				return A000120.name;
			case _A023416 :
				return A023416.name;
			case _A062756 :
				return A062756.name;
			case _A005590 :
				return A005590.name;
			case _A005812 :
				return A005812.name;
			case _A006287 :
				return A006287.name;
			case _A008687 :
				return A008687.name;
			case _A037834 :
				return A037834.name;
			case _A053735 :
				return A053735.name;
			case _A002487 :
				return A002487.name;
			case _A038760 :
				return A038760.name;
			case _A051734 :
				return A051734.name;
		}
		return ("");
	}

	public static String getDescription(int index) {
		switch (index) {
			case _ARAND :
				return ARand.description;
			case _A000120 :
				return A000120.description;
			case _A023416 :
				return A023416.description;
			case _A062756 :
				return A062756.description;
			case _A005590 :
				return A005590.description;
			case _A005812 :
				return A005812.description;
			case _A006287 :
				return A006287.description;
			case _A008687 :
				return A008687.description;
			case _A037834 :
				return A037834.description;
			case _A053735 :
				return A053735.description;
			case _A002487 :
				return A002487.description;
			case _A038760 :
				return A038760.description;
			case _A051734 :
				return A051734.description;
		}
		return ("");
	}

	public int getMode() {
		return (mode);
	}

	public void reset() {
		index = 0;
		computedNext = false;
		updateControlPanel();
	}

	public int next() {
		if (computedNext) {
			currentVal = nextVal;
			computedNext = false;
		} else if (size == 0) {
			currentVal = 0;
		} else {
			currentVal = sequence.getValue(index) % size;
		}
		updateControlPanel();
		long temp = index + 1;
		if (temp > index)
			index = temp;
		return (currentVal);
	}

	public void skipAhead() {
		long temp = sequence.skipAhead(index);
		if (temp > index)
			index = temp;
		computedNext = false;
	}

	public void goBack() {
		long temp = sequence.goBack(index);
		if (temp < index)
			index = temp;
		computedNext = false;
	}

	public int peekAhead() {
		if (!computedNext) {
			if (size == 0)
				nextVal = 0;
			else
				nextVal = sequence.getValue(index) % size;
			computedNext = true;
		}
		return (nextVal);
	}

	public int current() {
		return (currentVal);
	}

	public void updateControlPanel() {
		((MetaMix) parentFrame).controls.nLabel.setText(
			" [f(" + String.valueOf(index) + ") = " + currentVal + "]");
	}
}
